/*	Pace Clock on ATmega644
 *	ECE 4760 Spring 2011
 *	Final Project
 *
 * 	By Paul Swirhun and Andy Liang
 */

/*	Used to interface AVR microcontroller with
 *	TLC5940 LED driver chip from Texas Instruments
 */

//Define the port that communicates with the LED controllers
#define LED_PORT	PORTB
#define LED_PIN		PINB
#define LED_DDR		DDRB

//The bit wirings for the connections to the LED controllers
//The top 3 are certain MCU pins, and should not be changed
#define SCK		7	//pin 25 on TLC5940
#define MISO 	6	//not used
#define MOSI	5	//pin 26 on TLC5940
#define BLANK	4	//pin 23 on TLC5940
#define VPRG	3	//pin 27 on TLC5940
#define GSCLK	2	//pin 18 on TLC5940
#define XLAT	1	//pin 24 on TLC5940


//**********************************************************
/*	Send k bytes to LED controllers, starting with index (k-1) and going to (0)
	Sends MSB first (ie, from index 95 to 0). k is the number of bytes to send,
	and must be less than 127 in this implementation. Note that each TLC5940
	in a chain of them uses 24 bytes, so we will typically have k==96.
	The function latches the data in, pulses BLANK to reset counter, and 
	pulses GSCLK to give DC illumination.
 */
void send_TLC5940(unsigned char vals[],unsigned char k);

//**********************************************************
//Sets up the SPI interface for the LED controllers
void LED_init();

//**********************************************************
/*	Overwrites the LED_buffer_long, a 96 byte dataset to send to LED controllers.
	This function calls setup_LED_buffer

	digits[4] holds the digits in a 1-hot ABCDEFG code.
	note that digits[0] is the rightmost display digit.
	to display a time of 12:30, digits[] = {0,3,2,1}

	This function also applies the colon, hyphen, and period settings.
	
	If null_high ==1, then the 2 most significant panels are turned off
	if it won't change the time displayed on screen. This means digits[3] is
	turned off if it is zero, and digits[2] is turned off if digits[3] and
	digits[2] are both zero.
	
	If null_high ==2, then the 1 most siginficant panel is turned off
	if it won't change the time displayed on screen. This means digits[3] is
	turned off if it is zero.
	
	If null_high ==0, then no panels are turned off and they display the inputs.
	
	This is done by copying out to a local variable,
	so the input data array will not be modified. */
void write_LED_buffer(unsigned char digits[4], unsigned char colon, unsigned char null_high);


//**********************************************************
/*	Overwrites the LED_buffer_long with 96 bytes of octet (12-bit) brightness values */
void setup_LED_buffer(void);

//**********************************************************
/* Overwrites the LED_buffer_dot_correction vector (all equalized, at this time)*/
void setup_LED_buffer_dot_correction(void);
