/*	Used to interface AVR microcontroller with
 *	PCF8563 calendar/clock chip from NXP.
 *	I2C interface aka TWI on Atmel products provided externally
 *
 * 	By Paul Swirhun.
 */

#include "clockInterface.h"
#include "i2cmaster.h"

//I2C bus slave address: Read:A3, Write:A2

#define begin {
#define end }

//SDA is PORTC.1, SCL is PORTC.0

//Send default settings to the clock/calendar chip
void init_PCF8563(void)
begin
	i2c_start_wait(PCF_WRITE);		//start and write slave address
	i2c_write(Control_status_1);	//write register address
	i2c_write(0x00);	//write register data
	i2c_write(0x00);	//write the next register (Control_status_2)	
	i2c_stop();			

	i2c_start_wait(PCF_WRITE);
	i2c_write(CLKOUT_control);	//write register address
	i2c_write(((1<<FE_bit) | FD_set)); //activate CLKOUT at FD_set (0b11 = 1Hz)
	i2c_write(TD_set); //timer is disabled but set at TD_set (3 = 1/60Hz)
	i2c_stop();
end

/*	Combo setter/getter for any register, when you just want to use one
	Uses format of the timer chip (NOT BINARY! DEPENDENT ON THE REGISTER!)
	s=1 for set. s=0 for get.
	reg = the register address or name (use macros in header file)
	val = input byte for the register (ignored if doing a get)
 */
unsigned char PCF_register_setget(unsigned char s, unsigned char reg, unsigned char val)
begin
	unsigned char ret; //the value to return (==val if used as set)

	i2c_start_wait(PCF_WRITE);
	i2c_write(reg); //write the register address

	if (s) { //perform a set (write to register)
		i2c_write(val); //write the value in
		ret = val; //dummy return value
	} else {
		i2c_rep_start(PCF_READ); //switch to read-mode
		ret = i2c_readNak(); //read without acknowledge (signifies done reading)
	}

	i2c_stop(); //free up the TWI bus
	return ret;
end
