/*	Used to interface AVR microcontroller with
 *	PCF8563 calendar/clock chip from NXP.
 *	I2C interface aka TWI on Atmel products provided externally
 *
 * 	By Paul Swirhun.
 */

//For ATmega644, C.0 is TWI clock line
//and 			 C.1 is the TWI serial data line

#define	FD_set		0x02	//0x03 for a 32Hz CLKOUT output clock
#define TD_set		0x03	//0x01 for 1/60Hz timer source (not used)

#define PCF_WRITE	0xA2	//PCF8563 address for write operations
#define PCF_READ	0xA3	//PCF8563 address for read operations

#define begin {
#define end }

//Control and Status Registers of the PCF8563
#define Control_status_1	0x00	//Control_status_1
#define Control_status_2	0x01	//Control_status_2
#define CLKOUT_control		0x0D	//CLKOUT_control
#define	Timer_control		0x0E	//[7]:FE, [1:0]:FD
#define Timer				0x0F	//the coundown timer (8-bit, adjustable tick)

//Timing Registers
#define	VL_seconds		0x02	//[7]:VL [6:4]:tens of seconds (0 to 5) [3:0]:ones of seconds (0 to 9)
#define Minutes			0x03	//[6:4]:tens of minutes (0 to 5) [3:0]:ones of minutes (0 to 9)
#define Hours			0x04	//[5:4]:tens of hours (0 to 2) [3:0]:ones of hours (0 to 9)
#define Days			0x05	//[5:4]:tens of days (0 to 3) [3:0]:ones of days (0 to 9)
#define Weekdays		0x06	//[2:0] Weekdays (0 to 6) (Sunday = 0)
#define Century_months	0x07	//[7]:C [4]:tens of months [3:0]:ones of months (0 to 9) (january is 1)
#define Years			0x08	//[7:4]:tens of years [3:0]:ones of years (handles leap years)

//Alary Registers (setting bit7 to 0 ENABLES the alarm)
#define Minute_alarm	0x09	//[7]:AE_M [6:4]:tens of minutes (0 to 5) [3:0]:ones of minutes (0 to 9)
#define Hour_alarm		0x0A	//[7]:AE_H [5:4]:tens of hours (0 to 2) [3:0]:ones of hours (0 to 9)
#define Day_alarm		0x0B	//[7]:AE_D [5:4]:tens of days (0 to 3) [3:0]:ones of days (0 to 9)
#define Weekday_alarm	0x0C	//[7]:AE_W [2:0] Weekdays (0 to 6) (Sunday = 0)

//Bits of Register Control_status_1
#define TEST1	7	//set to 0 during normal operations
#define STOP	5	//RTC clock asynchronously zeroed and stopped when 1
#define TESTC	3	//Power-on Reset override enable (0:disabled in normal operation)

//Bits of Register Control_status_2
#define TI_TP	4	//sets behavior of TF wrt INT-not
#define AF		3	//Alarm flag = 1 when alarm
#define TF		2	//Timer flag = 1 at end of countdown
#define AIE		1	//1:enable alarm interrupt (interrupt output pin')
#define	TIE		0	//1:enable timer interrupt (interrupt output pin')

//Bits of Timing Registers
#define VL		7	//1:clock integrity not guaranteed (power outage in the past)
#define C		7
#define AE_M	7	//0:Alarm month enabled
#define AE_H	7	//0:Alarm hour enabled
#define AE_D	7	//0:Alarm day enabled
#define AE_W	7	//0:Alarm weekday enabled

//Other bits
#define FE_bit		7	//1:the CLKOUT output is activated
#define	TE_bit		7	//1:timer is enabled

//**********Functions**********//

//Send default settings to the clock/calendar chip
void init_PCF8563(void);	


/*	Combo setter/getter for any register, when you just want to use one
	Uses format of the timer chip (NOT BINARY! DEPENDENT ON THE REGISTER!)
	s=1 for set. s=0 for get.
	reg = the register address or name (use macros in header file)
	val = input byte for the register (ignored if doing a get) 	*/
unsigned char PCF_register_setget(unsigned char, unsigned char, unsigned char); //set or get register


